@@ -14,9 +14,6 @@ struct SDL_Surface;
 
 namespace BitmapHandler
 {
-	SDL_Surface * loadH3PCX(ui8 * data, size_t size);
-	//Load file from specific LOD
-	SDL_Surface * loadBitmapFromDir(std::string path, std::string fname, bool setKey=true);
-	//Load file from any LODs
+	//Load file from /DATA or /SPRITES
 	SDL_Surface * loadBitmap(std::string fname, bool setKey=true);
 }
