@@ -13,23 +13,12 @@
 #include <SDL_version.h>
 
 #if (SDL_MAJOR_VERSION == 2)
-#define VCMI_SDL2
 
 #include <SDL_keycode.h>
 typedef int SDLX_Coord;
 typedef int SDLX_Size;
 
-typedef SDL_Keycode SDLKey;
 
-#define SDL_SRCCOLORKEY SDL_TRUE
-
-#define SDL_FULLSCREEN SDL_WINDOW_FULLSCREEN
-
-#elif (SDL_MAJOR_VERSION == 1) 
-#define VCMI_SDL1
-//SDL 1.x
-typedef Sint16 SDLX_Coord;
-typedef Uint16 SDLX_Size;
 #else
 #error "unknown or unsupported SDL version"
 #endif
