@@ -209,10 +209,10 @@ void CButton::hover (bool on)
 		setState(PRESSED);*/
 
 	std::string name = hoverTexts[getState()].empty()
-		? hoverTexts[getState()]
-		: hoverTexts[0];
+		? hoverTexts[0]
+		: hoverTexts[getState()];
 
-	if(!name.empty() && !isBlocked()) //if there is no name, there is nohing to display also
+	if(!name.empty() && !isBlocked()) //if there is no name, there is nothing to display also
 	{
 		if (LOCPLINT && LOCPLINT->battleInt) //for battle buttons
 		{
@@ -272,10 +272,11 @@ void CButton::setIndex(size_t index, bool playerColoredButton)
 	if (index == currentImage || index>=imageNames.size())
 		return;
 	currentImage = index;
-	setImage(new CAnimation(imageNames[index]), playerColoredButton);
+	auto anim = std::make_shared<CAnimation>(imageNames[index]);
+	setImage(anim, playerColoredButton);
 }
 
-void CButton::setImage(CAnimation* anim, bool playerColoredButton, int animFlags)
+void CButton::setImage(std::shared_ptr<CAnimation> anim, bool playerColoredButton, int animFlags)
 {
 	OBJ_CONSTRUCTION_CAPTURING_ALL;
 
@@ -294,14 +295,9 @@ void CButton::setPlayerColor(PlayerColor player)
 void CButton::showAll(SDL_Surface * to)
 {
 	CIntObject::showAll(to);
-	
-	#ifdef VCMI_SDL1
-	if (borderColor && borderColor->unused == 0)
-		CSDL_Ext::drawBorder(to, pos.x-1, pos.y-1, pos.w+2, pos.h+2, int3(borderColor->r, borderColor->g, borderColor->b));
-	#else
+
 	if (borderColor && borderColor->a == 0)
 		CSDL_Ext::drawBorder(to, pos.x-1, pos.y-1, pos.w+2, pos.h+2, int3(borderColor->r, borderColor->g, borderColor->b));
-	#endif // 0
 }
 
 std::pair<std::string, std::string> CButton::tooltip()
@@ -424,8 +420,8 @@ void CToggleGroup::addToggle(int identifier, CToggleBase* bt)
 	buttons[identifier] = bt;
 }
 
-CToggleGroup::CToggleGroup(const CFunctionList<void(int)> &OnChange, bool musicLikeButtons)
-: onChange(OnChange), selectedID(-2), musicLike(musicLikeButtons)
+CToggleGroup::CToggleGroup(const CFunctionList<void(int)> &OnChange)
+: onChange(OnChange), selectedID(-2)
 {}
 
 void CToggleGroup::setSelected(int id)
@@ -452,26 +448,86 @@ void CToggleGroup::selectionChanged(int to)
 		parent->redraw();
 }
 
-void CToggleGroup::show(SDL_Surface * to)
+CVolumeSlider::CVolumeSlider(const Point &position, const std::string &defName, const int value,
+                             const std::pair<std::string, std::string> * const help) :
+	value(value),
+	helpHandlers(help)
 {
-	if (musicLike)
-	{
-		if (auto intObj = dynamic_cast<CIntObject*>(buttons[selectedID])) // hack-ish workagound to avoid diamond problem with inheritance
-			intObj->show(to);
-	}
-	else
-		CIntObject::show(to);
+	OBJ_CONSTRUCTION_CAPTURING_ALL;
+	animImage = new CAnimImage(std::make_shared<CAnimation>(defName), 0, 0, position.x, position.y),
+	assert(!defName.empty());
+	addUsedEvents(LCLICK | RCLICK | WHEEL);
+	pos.x += position.x;
+	pos.y += position.y;
+	pos.w = (animImage->pos.w + 1) * animImage->size();
+	pos.h = animImage->pos.h;
+	type |= REDRAW_PARENT;
+	setVolume(value);
 }
 
-void CToggleGroup::showAll(SDL_Surface * to)
+void CVolumeSlider::setVolume(int value_)
 {
-	if (musicLike)
+	value = value_;
+	moveTo(value * static_cast<double>(animImage->size()) / 100.0);
+}
+
+void CVolumeSlider::moveTo(int id)
+{
+	vstd::abetween(id, 0, animImage->size() - 1);
+	animImage->setFrame(id);
+	animImage->moveTo(Point(pos.x + (animImage->pos.w + 1) * id, pos.y));
+	if (active)
+		redraw();
+}
+
+void CVolumeSlider::addCallback(std::function<void(int)> callback)
+{
+	onChange += callback;
+}
+
+void CVolumeSlider::clickLeft(tribool down, bool previousState)
+{
+	if (down)
 	{
-		if (auto intObj = dynamic_cast<CIntObject*>(buttons[selectedID])) // hack-ish workagound to avoid diamond problem with inheritance
-			intObj->showAll(to);
+		double px = GH.current->motion.x - pos.x;
+		double rx = px / static_cast<double>(pos.w);
+		// setVolume is out of 100
+		setVolume(rx * 100);
+		// Volume config is out of 100, set to increments of 5ish roughly based on the half point of the indicator
+		// 0.0 -> 0, 0.05 -> 5, 0.09 -> 5,...,
+		// 0.1 -> 10, ..., 0.19 -> 15, 0.2 -> 20, ...,
+		// 0.28 -> 25, 0.29 -> 30, 0.3 -> 30, ...,
+		// 0.85 -> 85, 0.86 -> 90, ..., 0.87 -> 90,...,
+		// 0.95 -> 95, 0.96 -> 100, 0.99 -> 100
+		int volume = 5 * int(rx * (2 * animImage->size() + 1));
+		onChange(volume);
+	}
+}
+
+void CVolumeSlider::clickRight(tribool down, bool previousState)
+{
+	if (down)
+	{
+		double px = GH.current->motion.x - pos.x;
+		int index = px / static_cast<double>(pos.w) * animImage->size();
+		std::string hoverText = helpHandlers[index].first;
+		std::string helpBox = helpHandlers[index].second;
+		if(!helpBox.empty())
+			CRClickPopup::createAndPush(helpBox);
+		if(GH.statusbar)
+			GH.statusbar->setText(helpBox);
+	}
+}
+
+void CVolumeSlider::wheelScrolled(bool down, bool in)
+{
+	if (in)
+	{
+		int volume = value + 3 * (down ? 1 : -1);
+		vstd::abetween(volume, 0, 100);
+		setVolume(volume);
+		onChange(volume);
 	}
-	else
-		CIntObject::showAll(to);
 }
 
 void CSlider::sliderClicked()
