@@ -144,34 +144,34 @@ std::string CStackWindow::generateStackExpDescription()
 		tier = 0;
 	int number;
 	std::string expText = CGI->generaltexth->zcrexp[325];
-	boost::replace_first (expText, "%s", creature->namePl);
-	boost::replace_first (expText, "%s", CGI->generaltexth->zcrexp[rank]);
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(rank));
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(stack->experience));
+	boost::replace_first(expText, "%s", creature->namePl);
+	boost::replace_first(expText, "%s", CGI->generaltexth->zcrexp[rank]);
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(rank));
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(stack->experience));
 	number = CGI->creh->expRanks[tier][rank] - stack->experience;
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(number));
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(number));
 
 	number = CGI->creh->maxExpPerBattle[tier]; //percent
-	boost::replace_first (expText, "%i%", boost::lexical_cast<std::string>(number));
+	boost::replace_first(expText, "%i%", boost::lexical_cast<std::string>(number));
 	number *= CGI->creh->expRanks[tier].back() / 100; //actual amount
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(number));
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(number));
 
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(stack->count)); //Number of Creatures in stack
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(stack->count)); //Number of Creatures in stack
 
 	int expmin = std::max(CGI->creh->expRanks[tier][std::max(rank-1, 0)], (ui32)1);
 	number = (stack->count * (stack->experience - expmin)) / expmin; //Maximum New Recruits without losing current Rank
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(number)); //TODO
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(number)); //TODO
 
-	boost::replace_first (expText, "%.2f", boost::lexical_cast<std::string>(1)); //TODO Experience Multiplier
+	boost::replace_first(expText, "%.2f", boost::lexical_cast<std::string>(1)); //TODO Experience Multiplier
 	number = CGI->creh->expAfterUpgrade;
-	boost::replace_first (expText, "%.2f", boost::lexical_cast<std::string>(number) + "%"); //Upgrade Multiplier
+	boost::replace_first(expText, "%.2f", boost::lexical_cast<std::string>(number) + "%"); //Upgrade Multiplier
 
 	expmin = CGI->creh->expRanks[tier][9];
 	int expmax = CGI->creh->expRanks[tier][10];
 	number = expmax - expmin;
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(number)); //Experience after Rank 10
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(number)); //Experience after Rank 10
 	number = (stack->count * (expmax - expmin)) / expmin;
-	boost::replace_first (expText, "%i", boost::lexical_cast<std::string>(number)); //Maximum New Recruits to remain at Rank 10 if at Maximum Experience
+	boost::replace_first(expText, "%i", boost::lexical_cast<std::string>(number)); //Maximum New Recruits to remain at Rank 10 if at Maximum Experience
 
 	return expText;
 }
@@ -181,29 +181,10 @@ void CStackWindow::removeStackArtifact(ArtifactPosition pos)
 	auto art = info->stackNode->getArt(ArtifactPosition::CREATURE_SLOT);
 	LOCPLINT->cb->swapArtifacts(ArtifactLocation(info->stackNode, pos),
 								ArtifactLocation(info->owner, art->firstBackpackSlot(info->owner)));
-	delete stackArtifactButton;
-	delete stackArtifactHelp;
-	delete stackArtifactIcon;
-}
-
-void CStackWindow::setStackArtifact(const CArtifactInstance * art, Point artPos)
-{
-	if (art)
-	{
-		stackArtifactIcon = new CAnimImage("ARTIFACT", art->artType->iconIndex, 0, pos.x, pos.y);
-		stackArtifactHelp = new LRClickableAreaWTextComp(Rect(artPos, Point(44, 44)), CComponent::artifact);
-		stackArtifactHelp->type = art->artType->id;
-
-		const JsonNode & text = VLC->generaltexth->localizedTexts["creatureWindow"]["returnArtifact"];
-
-		if (info->owner)
-		{
-			stackArtifactButton = new CButton(Point(artPos.x - 2 , artPos.y + 46), "stackWindow/cancelButton",
-			                                  CButton::tooltip(text),
-			                                  [=]{ removeStackArtifact(ArtifactPosition::CREATURE_SLOT); });
-		}
-	}
-
+	stackArtifactButton.reset();
+	stackArtifactHelp.reset();
+	stackArtifactIcon.reset();
+	redraw();
 }
 
 void CStackWindow::CWindowSection::createStackInfo(bool showExp, bool showArt)
@@ -221,7 +202,7 @@ void CStackWindow::CWindowSection::createStackInfo(bool showExp, bool showArt)
 	if (parent->info->stackNode != nullptr && parent->info->commander == nullptr)
 	{
 		//normal stack, not a commander and not non-existing stack (e.g. recruitment dialog)
-		pic->setAmount(parent->info->stackNode->count);
+		pic->setAmount(parent->info->creatureCount);
 	}
 
 	std::string visibleName;
@@ -236,11 +217,6 @@ void CStackWindow::CWindowSection::createStackInfo(bool showExp, bool showArt)
 		dmgMultiply += parent->info->owner->Attack();
 
 	new CPicture("stackWindow/icons", 117, 32);
-	printStatBase(EStat::ATTACK, CGI->generaltexth->primarySkillNames[0], parent->info->creature->valOfBonuses(Bonus::PRIMARY_SKILL, PrimarySkill::ATTACK), parent->info->stackNode->Attack());
-	printStatBase(EStat::DEFENCE, CGI->generaltexth->primarySkillNames[1], parent->info->creature->valOfBonuses(Bonus::PRIMARY_SKILL, PrimarySkill::DEFENSE), parent->info->stackNode->Defense());
-	printStatRange(EStat::DAMAGE, CGI->generaltexth->allTexts[199], parent->info->stackNode->getMinDamage() * dmgMultiply, parent->info->stackNode->getMaxDamage() * dmgMultiply);
-	printStatBase(EStat::HEALTH, CGI->generaltexth->allTexts[388], parent->info->creature->valOfBonuses(Bonus::STACK_HEALTH), parent->info->stackNode->valOfBonuses(Bonus::STACK_HEALTH));
-	printStatBase(EStat::SPEED, CGI->generaltexth->zelp[441].first, parent->info->creature->Speed(), parent->info->stackNode->Speed());
 
 	const CStack * battleStack = parent->info->stack;
 	bool shooter = parent->info->stackNode->hasBonusOfType(Bonus::SHOOTER) && parent->info->stackNode->valOfBonuses(Bonus::SHOTS);
@@ -248,6 +224,12 @@ void CStackWindow::CWindowSection::createStackInfo(bool showExp, bool showArt)
 
 	if (battleStack != nullptr) // in battle
 	{
+		printStatBase(EStat::ATTACK, CGI->generaltexth->primarySkillNames[0], parent->info->creature->Attack(), battleStack->Attack());
+		printStatBase(EStat::DEFENCE, CGI->generaltexth->primarySkillNames[1], parent->info->creature->Defense(false), battleStack->Defense());
+		printStatRange(EStat::DAMAGE, CGI->generaltexth->allTexts[199], parent->info->stackNode->getMinDamage() * dmgMultiply, battleStack->getMaxDamage() * dmgMultiply);
+		printStatBase(EStat::HEALTH, CGI->generaltexth->allTexts[388], parent->info->creature->valOfBonuses(Bonus::STACK_HEALTH), battleStack->valOfBonuses(Bonus::STACK_HEALTH));
+		printStatBase(EStat::SPEED, CGI->generaltexth->zelp[441].first, parent->info->creature->Speed(), battleStack->Speed());
+
 		if (shooter)
 			printStatBase(EStat::SHOTS, CGI->generaltexth->allTexts[198], battleStack->valOfBonuses(Bonus::SHOTS), battleStack->shots);
 		if (caster)
@@ -256,6 +238,12 @@ void CStackWindow::CWindowSection::createStackInfo(bool showExp, bool showArt)
 	}
 	else
 	{
+		printStatBase(EStat::ATTACK, CGI->generaltexth->primarySkillNames[0], parent->info->creature->Attack(), parent->info->stackNode->Attack());
+		printStatBase(EStat::DEFENCE, CGI->generaltexth->primarySkillNames[1], parent->info->creature->Defense(false), parent->info->stackNode->Defense());
+		printStatRange(EStat::DAMAGE, CGI->generaltexth->allTexts[199], parent->info->stackNode->getMinDamage() * dmgMultiply, parent->info->stackNode->getMaxDamage() * dmgMultiply);
+		printStatBase(EStat::HEALTH, CGI->generaltexth->allTexts[388], parent->info->creature->valOfBonuses(Bonus::STACK_HEALTH), parent->info->stackNode->valOfBonuses(Bonus::STACK_HEALTH));
+		printStatBase(EStat::SPEED, CGI->generaltexth->zelp[441].first, parent->info->creature->Speed(), parent->info->stackNode->Speed());
+
 		if (shooter)
 			printStat(EStat::SHOTS, CGI->generaltexth->allTexts[198], parent->info->stackNode->valOfBonuses(Bonus::SHOTS));
 		if (caster)
@@ -267,12 +255,6 @@ void CStackWindow::CWindowSection::createStackInfo(bool showExp, bool showArt)
 	auto luck = new MoraleLuckBox(false, genRect(42, 42, 375, 110));
 	luck->set(parent->info->stackNode);
 
-	if (showArt)
-	{
-		Point pos = showExp ? Point(375, 32) : Point(347, 32);
-		parent->setStackArtifact(parent->info->stackNode->getArt(ArtifactPosition::CREATURE_SLOT), pos);
-	}
-
 	if (showExp)
 	{
 		const CStackInstance * stack = parent->info->stackNode;
@@ -280,22 +262,60 @@ void CStackWindow::CWindowSection::createStackInfo(bool showExp, bool showArt)
 		if (parent->info->commander)
 		{
 			const CCommanderInstance * commander = parent->info->commander;
-			new CAnimImage("PSKIL42", 4, 0, pos.x, pos.y); // experience icon
+			parent->expRankIcon = new CAnimImage("PSKIL42", 4, 0, pos.x, pos.y); // experience icon
 
-			auto expArea = new LRClickableAreaWTextComp(Rect(pos.x, pos.y, 44, 44), CComponent::experience);
-			expArea->text = CGI->generaltexth->allTexts[2];
-			expArea->bonusValue = commander->getExpRank();
-			boost::replace_first(expArea->text, "%d", boost::lexical_cast<std::string>(commander->getExpRank()));
-			boost::replace_first(expArea->text, "%d", boost::lexical_cast<std::string>(CGI->heroh->reqExp(commander->getExpRank()+1)));
-			boost::replace_first(expArea->text, "%d", boost::lexical_cast<std::string>(commander->experience));
+			parent->expArea = new LRClickableAreaWTextComp(Rect(
+					pos.x, pos.y, 44, 44), CComponent::experience);
+			parent->expArea->text = CGI->generaltexth->allTexts[2];
+			reinterpret_cast<LRClickableAreaWTextComp*>(parent->expArea)->bonusValue =
+					commander->getExpRank();
+			boost::replace_first(parent->expArea->text, "%d",
+								 boost::lexical_cast<std::string>(commander->getExpRank()));
+			boost::replace_first(parent->expArea->text, "%d",
+								 boost::lexical_cast<std::string>(CGI->heroh->reqExp(commander->getExpRank() + 1)));
+			boost::replace_first(parent->expArea->text, "%d",
+								 boost::lexical_cast<std::string>(commander->experience));
 		}
 		else
 		{
-			new CAnimImage("stackWindow/levels", stack->getExpRank(), 0, pos.x, pos.y);
-			auto expArea = new LRClickableAreaWText(Rect(pos.x, pos.y, 44, 44));
-			expArea->text = parent->generateStackExpDescription();
+			parent->expRankIcon = new CAnimImage(
+					"stackWindow/levels", stack->getExpRank(), 0, pos.x, pos.y);
+			parent->expArea = new LRClickableAreaWText(Rect(pos.x, pos.y, 44, 44));
+			parent->expArea->text = parent->generateStackExpDescription();
+		}
+		parent->expLabel = new CLabel(
+				pos.x + 21, pos.y + 52, FONT_SMALL, CENTER, Colors::WHITE,
+				makeNumberShort<TExpType>(stack->experience, 6));
+	}
+
+	if (showArt)
+	{
+		Point pos = showExp ? Point(375, 32) : Point(347, 32);
+		// ALARMA: do not refactor this into a separate function
+		// otherwise, artifact icon is drawn near the hero's portrait
+		// this is really strange
+		auto art = parent->info->stackNode->getArt(ArtifactPosition::CREATURE_SLOT);
+		if (art)
+		{
+			parent->stackArtifactIcon.reset(new CAnimImage(
+					"ARTIFACT", art->artType->iconIndex, 0, pos.x, pos.y));
+			parent->stackArtifactIcon->recActions &= ~DISPOSE;
+			parent->stackArtifactHelp.reset(new LRClickableAreaWTextComp(Rect(
+					pos, Point(44, 44)), CComponent::artifact));
+			parent->stackArtifactHelp->recActions &= ~DISPOSE;
+			parent->stackArtifactHelp->type = art->artType->id;
+			const JsonNode & text =
+					VLC->generaltexth->localizedTexts["creatureWindow"]["returnArtifact"];
+
+			if (parent->info->owner)
+			{
+				parent->stackArtifactButton.reset(new CButton(
+						Point(pos.x - 2 , pos.y + 46), "stackWindow/cancelButton",
+						CButton::tooltip(text),
+						[=]{ parent->removeStackArtifact(ArtifactPosition::CREATURE_SLOT); }));
+				parent->stackArtifactButton->recActions &= ~DISPOSE;
+			}
 		}
-		new CLabel(pos.x + 21, pos.y + 52, FONT_SMALL, CENTER, Colors::WHITE, makeNumberShort<TExpType>(stack->experience, 6));
 	}
 }
 
@@ -317,19 +337,19 @@ void CStackWindow::CWindowSection::createActiveSpells()
 	for(si32 effect : spells)
 	{
 		const CSpell * sp = CGI->spellh->objects[effect];
-		
+
 		std::string spellText;
-		
+
 		//not all effects have graphics (for eg. Acid Breath)
 		//for modded spells iconEffect is added to SpellInt.def
-		const bool hasGraphics = (effect < SpellID::THUNDERBOLT) || (effect >= SpellID::AFTER_LAST); 
-		
-		if (hasGraphics) 
+		const bool hasGraphics = (effect < SpellID::THUNDERBOLT) || (effect >= SpellID::AFTER_LAST);
+
+		if (hasGraphics)
 		{
 			spellText = CGI->generaltexth->allTexts[610]; //"%s, duration: %d rounds."
-			boost::replace_first (spellText, "%s", sp->name);
+			boost::replace_first(spellText, "%s", sp->name);
 			int duration = battleStack->getBonusLocalFirst(Selector::source(Bonus::SPELL_EFFECT,effect))->turnsRemain;
-			boost::replace_first (spellText, "%d", boost::lexical_cast<std::string>(duration));
+			boost::replace_first(spellText, "%d", boost::lexical_cast<std::string>(duration));
 
 			new CAnimImage("SpellInt", effect + 1, 0, firstPos.x + offset.x * printed, firstPos.y + offset.y * printed);
 			new LRClickableAreaWText(Rect(firstPos + offset * printed, Point(50, 38)), spellText, spellText);
@@ -440,7 +460,7 @@ CIntObject * CStackWindow::createSkillEntry(int index)
 	{
 		if (index == 0 && skillID >= 100)
 		{
-			const Bonus *bonus = CGI->creh->skillRequirements[skillID-100].first;
+			const auto bonus = CGI->creh->skillRequirements[skillID-100].first;
 			const CStackInstance *stack = info->commander;
 			CClickableObject * icon = new CClickableObject(new CPicture(stack->bonusToGraphics(bonus)), []{});
 			icon->callback = [=]
@@ -573,14 +593,17 @@ void CStackWindow::CWindowSection::createButtonPanel()
 				{
 					resComps.push_back(new CComponent(CComponent::resource, i->resType, i->resVal));
 				}
-				LOCPLINT->showYesNoDialog(CGI->generaltexth->allTexts[207], onUpgrade, nullptr, true, resComps);
+
+				if(LOCPLINT->cb->getResourceAmount().canAfford(totalCost))
+				{
+					LOCPLINT->showYesNoDialog(CGI->generaltexth->allTexts[207], onUpgrade, nullptr, true, resComps);
+				}
+				else
+					LOCPLINT->showInfoDialog(CGI->generaltexth->allTexts[314], resComps);
 			};
 			auto upgradeBtn = new CButton(Point(221 + i * 40, 5), "stackWindow/upgradeButton", CGI->generaltexth->zelp[446], onClick, SDLK_1);
 
 			upgradeBtn->addOverlay(new CAnimImage("CPRSMALL", VLC->creh->creatures[upgradeInfo.info.newID[i]]->iconIndex));
-
-			if (!LOCPLINT->cb->getResourceAmount().canAfford(totalCost))
-				upgradeBtn->block(true);
 		}
 	}
 
@@ -741,19 +764,18 @@ void CStackWindow::initSections()
 void CStackWindow::initBonusesList()
 {
 	BonusList output, input;
-	input = *(info->stackNode->getBonuses(Selector::durationType(Bonus::PERMANENT).And(Selector::anyRange())));
+	input = *(info->stackNode->getBonuses(CSelector(Bonus::Permanent), Selector::all));
 
 	while (!input.empty())
 	{
-		Bonus * b = input.front();
-
-		output.push_back(new Bonus(*b));
+		auto b = input.front();
+		output.push_back(std::make_shared<Bonus>(*b));
 		output.back()->val = input.valOfBonuses(Selector::typeSubtype(b->type, b->subtype)); //merge multiple bonuses into one
 		input.remove_if (Selector::typeSubtype(b->type, b->subtype)); //remove used bonuses
 	}
 
 	BonusInfo bonusInfo;
-	for(Bonus* b : output)
+	for(auto b : output)
 	{
 		bonusInfo.name = info->stackNode->bonusToString(b, false);
 		bonusInfo.description = info->stackNode->bonusToString(b, true);
@@ -771,12 +793,12 @@ void CStackWindow::initBonusesList()
 	if (magicResistance)
 	{
 		BonusInfo bonusInfo;
-		Bonus b;
-		b.type = Bonus::MAGIC_RESISTANCE;
+		auto b = std::make_shared<Bonus>();
+		b->type = Bonus::MAGIC_RESISTANCE;
 
-		bonusInfo.name = VLC->getBth()->bonusToString(&b, info->stackNode, false);
-		bonusInfo.description = VLC->getBth()->bonusToString(&b, info->stackNode, true);
-		bonusInfo.imagePath = info->stackNode->bonusToGraphics(&b);
+		bonusInfo.name = VLC->getBth()->bonusToString(b, info->stackNode, false);
+		bonusInfo.description = VLC->getBth()->bonusToString(b, info->stackNode, true);
+		bonusInfo.imagePath = info->stackNode->bonusToGraphics(b);
 		activeBonuses.push_back(bonusInfo);
 	}
 }
@@ -786,10 +808,6 @@ void CStackWindow::init()
 	if (!info->stackNode)
 		info->stackNode = new CStackInstance(info->creature, 1);// FIXME: free data
 
-	stackArtifactHelp = nullptr;
-	stackArtifactIcon = nullptr;
-	stackArtifactButton = nullptr;
-
 	selectedIcon = nullptr;
 	selectedSkill = 0;
 	if (info->levelupInfo)
@@ -831,7 +849,7 @@ CStackWindow::CStackWindow(const CStackInstance * stack, bool popup):
 	info->creature = stack->type;
 	info->creatureCount = stack->count;
 	info->popupWindow = popup;
-	info->owner = dynamic_cast<const CGHeroInstance *> (stack->armyObj);	
+	info->owner = dynamic_cast<const CGHeroInstance *> (stack->armyObj);
 	init();
 }
 
@@ -861,7 +879,7 @@ CStackWindow::CStackWindow(const CCommanderInstance * commander, bool popup):
 	info->commander = commander;
 	info->creatureCount = 1;
 	info->popupWindow = popup;
-	info->owner = dynamic_cast<const CGHeroInstance *> (commander->armyObj);	
+	info->owner = dynamic_cast<const CGHeroInstance *> (commander->armyObj);
 	init();
 }
 
@@ -876,7 +894,7 @@ CStackWindow::CStackWindow(const CCommanderInstance * commander, std::vector<ui3
 	info->levelupInfo = StackWindowInfo::CommanderLevelInfo();
 	info->levelupInfo->skills = skills;
 	info->levelupInfo->callback = callback;
-	info->owner = dynamic_cast<const CGHeroInstance *> (commander->armyObj);		
+	info->owner = dynamic_cast<const CGHeroInstance *> (commander->armyObj);
 	init();
 }
 
