@@ -17,4 +17,13 @@ inline QString pathToQString(const boost::filesystem::path & path)
 #else
 	return QString::fromStdString(path.string());
 #endif
-}
\ No newline at end of file
+}
+
+inline boost::filesystem::path qstringToPath(const QString & path)
+{
+#ifdef VCMI_WINDOWS
+	return boost::filesystem::path(path.toStdWString());
+#else
+	return boost::filesystem::path(path.toUtf8().data());
+#endif
+}
