@@ -1,5 +1,7 @@
 #pragma once
 
+#include "../StdInc.h"
+
 namespace Ui {
 	class CSettingsView;
 }
@@ -7,13 +9,14 @@ namespace Ui {
 class CSettingsView : public QWidget
 {
 	Q_OBJECT
-	
+
 public:
 	explicit CSettingsView(QWidget *parent = 0);
 	~CSettingsView();
 
 	void loadSettings();
-	
+	void setDisplayList(const QStringList& displayList);
+
 private slots:
 	void on_comboBoxResolution_currentIndexChanged(const QString &arg1);
 
@@ -39,6 +42,10 @@ private slots:
 
 	void on_changeGameDataDir_clicked();
 
+	void on_comboBoxAutoCheck_currentIndexChanged(int index);
+
+	void on_comboBoxDisplayIndex_currentIndexChanged(int index);
+
 private:
 	Ui::CSettingsView *ui;
 };
