@@ -17,3 +17,8 @@ void IHandlerBase::registerObject(std::string scope, std::string type_name, std:
 {
 	return VLC->modh->identifiers.registerObject(scope, type_name, name, index);
 }
+
+std::string IHandlerBase::normalizeIdentifier(const std::string& scope, const std::string& remoteScope, const std::string& identifier) const
+{
+	return VLC->modh->normalizeIdentifier(scope, remoteScope, identifier);
+}
