@@ -34,11 +34,11 @@ CBank::~CBank()
 {
 }
 
-void CBank::initObj()
+void CBank::initObj(CRandomGenerator & rand)
 {
 	daycounter = 0;
 	resetDuration = 0;
-	VLC->objtypeh->getHandlerFor(ID, subID)->configureObject(this, cb->gameState()->getRandomGenerator());
+	VLC->objtypeh->getHandlerFor(ID, subID)->configureObject(this, rand);
 }
 
 std::string CBank::getHoverText(PlayerColor player) const
@@ -64,7 +64,8 @@ void CBank::setPropertyDer (ui8 what, ui32 val)
 				daycounter+=val;
 			break;
 		case ObjProperty::BANK_RESET:
-			initObj();
+			// FIXME: Object reset must be done by separate netpack from server
+			initObj(cb->gameState()->getRandomGenerator());
 			daycounter = 1; //yes, 1 since "today" daycounter won't be incremented
 			break;
 		case ObjProperty::BANK_CLEAR:
@@ -73,7 +74,7 @@ void CBank::setPropertyDer (ui8 what, ui32 val)
 	}
 }
 
-void CBank::newTurn() const
+void CBank::newTurn(CRandomGenerator & rand) const
 {
 	if (bc == nullptr)
 	{
@@ -250,18 +251,26 @@ void CBank::doVisit(const CGHeroInstance * hero) const
 			cb->showInfoDialog(&iw);
 		}
 
+		loot.clear();
+		iw.components.clear();
+		iw.text.clear();
+
 		if (!bc->spells.empty())
 		{
 			std::set<SpellID> spells;
 
 			bool noWisdom = false;
-			for (SpellID spell : bc->spells)
+			for(const SpellID & spellId : bc->spells)
 			{
-				iw.text.addTxt (MetaString::SPELL_NAME, spell);
-				if (VLC->spellh->objects[spell]->level <= hero->getSecSkillLevel(SecondarySkill::WISDOM) + 2)
+				const CSpell * spell = spellId.toSpell();
+				iw.text.addTxt (MetaString::SPELL_NAME, spellId);
+				if(spell->level <= hero->getSecSkillLevel(SecondarySkill::WISDOM) + 2)
 				{
-					spells.insert(spell);
-					iw.components.push_back(Component (Component::SPELL, spell, 0, 0));
+					if(hero->canLearnSpell(spell))
+					{
+						spells.insert(spellId);
+						iw.components.push_back(Component (Component::SPELL, spellId, 0, 0));
+					}
 				}
 				else
 					noWisdom = true;
@@ -271,10 +280,14 @@ void CBank::doVisit(const CGHeroInstance * hero) const
 				iw.text.addTxt (MetaString::ADVOB_TXT, 109); //no spellbook
 			else if (noWisdom)
 				iw.text.addTxt (MetaString::ADVOB_TXT, 108); //no expert Wisdom
-			if (spells.empty())
+
+			if(!iw.components.empty() || !iw.text.toString().empty())
+				cb->showInfoDialog(&iw);
+
+			if(!spells.empty())
 				cb->changeSpells (hero, true, spells);
 		}
-		loot.clear();
+
 		iw.components.clear();
 		iw.text.clear();
 
@@ -292,9 +305,9 @@ void CBank::doVisit(const CGHeroInstance * hero) const
 			loot.addReplacement(*elem.second);
 		}
 
-		if (ourArmy.Slots().size())
+		if(ourArmy.stacksCount())
 		{
-			if (ourArmy.Slots().size() == 1 && ourArmy.Slots().begin()->second->count == 1)
+			if(ourArmy.stacksCount() == 1 && ourArmy.Slots().begin()->second->count == 1)
 				iw.text.addTxt (MetaString::ADVOB_TXT, 185);
 			else
 				iw.text.addTxt (MetaString::ADVOB_TXT, 186);
