@@ -34,7 +34,7 @@ static void showInfoDialog(const CGHeroInstance* h, const ui32 txtID, const ui16
 	showInfoDialog(playerID,txtID,soundID);
 }
 
-void CGPandoraBox::initObj()
+void CGPandoraBox::initObj(CRandomGenerator & rand)
 {
 	blockVisit = (ID==Obj::PANDORAS_BOX); //block only if it's really pandora's box (events also derive from that class)
 	hasGuardians = stacks.size();
@@ -51,7 +51,7 @@ void CGPandoraBox::onHeroVisit(const CGHeroInstance * h) const
 
 void CGPandoraBox::giveContentsUpToExp(const CGHeroInstance *h) const
 {
-	cb->removeAfterVisit(this);
+	afterSuccessfulVisit();
 
 	InfoWindow iw;
 	iw.player = h->getOwner();
@@ -66,7 +66,25 @@ void CGPandoraBox::giveContentsUpToExp(const CGHeroInstance *h) const
 		}
 	}
 
-	if(gainedExp || changesPrimSkill || abilities.size())
+	std::vector<std::pair<SecondarySkill, ui8>> unpossessedAbilities; //ability + ability level
+	int abilitiesRequiringSlot = 0;
+
+	//filter out unnecessary secondary skills
+	for (int i = 0; i < abilities.size(); i++)
+	{
+		int curLev = h->getSecSkillLevel(abilities[i]);
+		bool abilityCanUseSlot = !curLev && ((h->secSkills.size() + abilitiesRequiringSlot) < GameConstants::SKILL_PER_HERO); //limit new abilities to number of slots
+
+		if (abilityCanUseSlot)
+			abilitiesRequiringSlot++;
+
+		if ((curLev && curLev < abilityLevels[i]) || abilityCanUseSlot)
+		{
+			unpossessedAbilities.push_back({ abilities[i], abilityLevels[i] });
+		}
+	}
+
+	if(gainedExp || changesPrimSkill || unpossessedAbilities.size())
 	{
 		TExpType expVal = h->calculateXp(gainedExp);
 		//getText(iw,afterBattle,175,h); //wtf?
@@ -75,25 +93,21 @@ void CGPandoraBox::giveContentsUpToExp(const CGHeroInstance *h) const
 
 		if(expVal)
 			iw.components.push_back(Component(Component::EXPERIENCE,0,expVal,0));
+
 		for(int i=0; i<primskills.size(); i++)
 			if(primskills[i])
 				iw.components.push_back(Component(Component::PRIM_SKILL,i,primskills[i],0));
 
-		for(int i=0; i<abilities.size(); i++)
-			iw.components.push_back(Component(Component::SEC_SKILL,abilities[i],abilityLevels[i],0));
+		for(auto abilityData : unpossessedAbilities)
+			iw.components.push_back(Component(Component::SEC_SKILL, abilityData.first, abilityData.second, 0));
 
 		cb->showInfoDialog(&iw);
 
 		//give sec skills
-		for(int i=0; i<abilities.size(); i++)
-		{
-			int curLev = h->getSecSkillLevel(abilities[i]);
+		for (auto abilityData : unpossessedAbilities)
+			cb->changeSecSkill(h, abilityData.first, abilityData.second, true);
 
-			if( (curLev  &&  curLev < abilityLevels[i])	|| (h->canLearnSkill() ))
-			{
-				cb->changeSecSkill(h,abilities[i],abilityLevels[i],true);
-			}
-		}
+		assert(h->secSkills.size() <= GameConstants::SKILL_PER_HERO);
 
 		//give prim skills
 		for(int i=0; i<primskills.size(); i++)
@@ -106,6 +120,7 @@ void CGPandoraBox::giveContentsUpToExp(const CGHeroInstance *h) const
 		if(expVal)
 			cb->changePrimSkill(h, PrimarySkill::EXPERIENCE, expVal, false);
 	}
+	//else { } //TODO:Create information that box was empty for now, and deliver to CGPandoraBox::giveContentsAfterExp or refactor
 
 	if(!cb->isVisitCoveredByAnotherQuery(this, h))
 		giveContentsAfterExp(h);
@@ -120,32 +135,45 @@ void CGPandoraBox::giveContentsAfterExp(const CGHeroInstance *h) const
 	InfoWindow iw;
 	iw.player = h->getOwner();
 
+	//TODO: reuse this code for Scholar skill
 	if(spells.size())
 	{
 		std::set<SpellID> spellsToGive;
-		iw.components.clear();
-		if (spells.size() > 1)
+
+		auto i = spells.cbegin();
+		while (i != spells.cend())
 		{
-			iw.text.addTxt(MetaString::ADVOB_TXT, 188); //%s learns spells
-		}
-		else
-		{
-			iw.text.addTxt(MetaString::ADVOB_TXT, 184); //%s learns a spell
-		}
-		iw.text.addReplacement(h->name);
-		std::vector<ConstTransitivePtr<CSpell> > * sp = &VLC->spellh->objects;
-		for(auto i=spells.cbegin(); i != spells.cend(); i++)
-		{
-			if ((*sp)[*i]->level <= h->getSecSkillLevel(SecondarySkill::WISDOM) + 2) //enough wisdom
+			iw.components.clear();
+			iw.text.clear();
+			spellsToGive.clear();
+
+			for (; i != spells.cend(); i++)
 			{
-				iw.components.push_back(Component(Component::SPELL,*i,0,0));
-				spellsToGive.insert(*i);
+				const CSpell * sp = (*i).toSpell();
+				if(h->canLearnSpell(sp))
+				{
+					iw.components.push_back(Component(Component::SPELL, *i, 0, 0));
+					spellsToGive.insert(*i);
+				}
+				if(spellsToGive.size() == 8) //display up to 8 spells at once
+				{
+					break;
+				}
+			}
+			if (!spellsToGive.empty())
+			{
+				if (spellsToGive.size() > 1)
+				{
+					iw.text.addTxt(MetaString::ADVOB_TXT, 188); //%s learns spells
+				}
+				else
+				{
+					iw.text.addTxt(MetaString::ADVOB_TXT, 184); //%s learns a spell
+				}
+				iw.text.addReplacement(h->name);
+				cb->changeSpells(h, true, spellsToGive);
+				cb->showInfoDialog(&iw);
 			}
-		}
-		if(!spellsToGive.empty())
-		{
-			cb->changeSpells(h,true,spellsToGive);
-			cb->showInfoDialog(&iw);
 		}
 	}
 
@@ -235,7 +263,7 @@ void CGPandoraBox::giveContentsAfterExp(const CGHeroInstance *h) const
 	iw.components.clear();
 	iw.text.clear();
 
-	if (creatures.Slots().size())
+	if(creatures.stacksCount())
 	{ //this part is taken straight from creature bank
 		MetaString loot;
 		for(auto & elem : creatures.Slots())
@@ -245,7 +273,7 @@ void CGPandoraBox::giveContentsAfterExp(const CGHeroInstance *h) const
 			loot.addReplacement(*elem.second);
 		}
 
-		if (creatures.Slots().size() == 1 && creatures.Slots().begin()->second->count == 1)
+		if(creatures.stacksCount() == 1 && creatures.Slots().begin()->second->count == 1)
 			iw.text.addTxt(MetaString::ADVOB_TXT, 185);
 		else
 			iw.text.addTxt(MetaString::ADVOB_TXT, 186);
@@ -254,7 +282,7 @@ void CGPandoraBox::giveContentsAfterExp(const CGHeroInstance *h) const
 		iw.text.addReplacement(h->name);
 
 		cb->showInfoDialog(&iw);
-		cb->giveCreatures(this, h, creatures, true);
+		cb->giveCreatures(this, h, creatures, false);
 	}
 	if(!hasGuardians && msg.size())
 	{
@@ -295,25 +323,25 @@ void CGPandoraBox::getText( InfoWindow &iw, bool &afterBattle, int val, int nega
 
 void CGPandoraBox::battleFinished(const CGHeroInstance *hero, const BattleResult &result) const
 {
-	if(result.winner)
-		return;
-
-	giveContentsUpToExp(hero);
+	if(result.winner == 0)
+	{
+		giveContentsUpToExp(hero);
+	}
 }
 
 void CGPandoraBox::blockingDialogAnswered(const CGHeroInstance *hero, ui32 answer) const
 {
-	if (answer)
+	if(answer)
 	{
-		if (stacksCount() > 0) //if pandora's box is protected by army
+		if(stacksCount() > 0) //if pandora's box is protected by army
 		{
 			showInfoDialog(hero,16,0);
 			cb->startBattleI(hero, this); //grants things after battle
 		}
-		else if (message.size() == 0 && resources.size() == 0
+		else if(message.size() == 0 && resources.size() == 0
 			&& primskills.size() == 0 && abilities.size() == 0
-			&& abilityLevels.size() == 0 &&  artifacts.size() == 0
-			&& spells.size() == 0 && creatures.Slots().size() > 0
+			&& abilityLevels.size() == 0 && artifacts.size() == 0
+			&& spells.size() == 0 && creatures.stacksCount() > 0
 			&& gainedExp == 0 && manaDiff == 0 && moraleDiff == 0 && luckDiff == 0) //if it gives nothing without battle
 		{
 			showInfoDialog(hero,15,0);
@@ -331,6 +359,11 @@ void CGPandoraBox::heroLevelUpDone(const CGHeroInstance *hero) const
 	giveContentsAfterExp(hero);
 }
 
+void CGPandoraBox::afterSuccessfulVisit() const
+{
+	cb->removeAfterVisit(this);
+}
+
 void CGEvent::onHeroVisit( const CGHeroInstance * h ) const
 {
 	if(!(availableFor & (1 << h->tempOwner.getNum())))
@@ -362,3 +395,13 @@ void CGEvent::activated( const CGHeroInstance * h ) const
 		giveContentsUpToExp(h);
 	}
 }
+
+void CGEvent::afterSuccessfulVisit() const
+{
+	if(removeAfterVisit)
+	{
+		cb->removeAfterVisit(this);
+	}
+	else if(hasGuardians)
+		hasGuardians = false;
+}
