@@ -1,6 +1,6 @@
 /*
  *
- * CRewardableObject.cpp, part of VCMI engine
+ * JsonRandom.cpp, part of VCMI engine
  *
  * Authors: listed in file AUTHORS in main folder
  *
@@ -133,10 +133,10 @@ namespace JsonRandom
 		if (value["type"].getType() == JsonNode::DATA_STRING)
 			return SpellID(VLC->modh->identifiers.getIdentifier("spell", value["type"]).get());
 
-		spells.erase(std::remove_if(spells.begin(), spells.end(), [=](SpellID spell)
+		vstd::erase_if(spells, [=](SpellID spell)
 		{
 			return VLC->spellh->objects[spell]->level != si32(value["level"].Float());
-		}), spells.end());
+		});
 
 		return SpellID(*RandomGeneratorUtil::nextItem(spells, rng));
 	}
@@ -219,9 +219,8 @@ namespace JsonRandom
 		std::vector<Bonus> ret;
 		for (const JsonNode & entry : value.Vector())
 		{
-			Bonus * bonus = JsonUtils::parseBonus(entry);
+			auto bonus = JsonUtils::parseBonus(entry);
 			ret.push_back(*bonus);
-			delete bonus;
 		}
 		return ret;
 	}
