@@ -8,14 +8,26 @@
 #include "../CTownHandler.h"
 #include "../StringConstants.h"
 #include "../filesystem/Filesystem.h"
-#include "CRmgTemplate.h"
-#include "CRmgTemplateZone.h"
 #include "CZonePlacer.h"
 #include "../mapObjects/CObjectClassesHandler.h"
 
+static const int3 dirs4[] = {int3(0,1,0),int3(0,-1,0),int3(-1,0,0),int3(+1,0,0)};
+
 void CMapGenerator::foreach_neighbour(const int3 &pos, std::function<void(int3& pos)> foo)
 {
-	for(const int3 &dir : dirs)
+	for(const int3 &dir : int3::getDirs())
+	{
+		int3 n = pos + dir;
+		/*important notice: perform any translation before this function is called,
+		so the actual map position is checked*/
+		if(map->isInTheMap(n))
+			foo(n);
+	}
+}
+
+void CMapGenerator::foreachDirectNeighbour(const int3& pos, std::function<void(int3& pos)> foo)
+{
+	for(const int3 &dir : dirs4)
 	{
 		int3 n = pos + dir;
 		if(map->isInTheMap(n))
@@ -46,6 +58,8 @@ void CMapGenerator::initTiles()
 			tiles[i][j] = new CTileInfo[level];
 		}
 	}
+
+	zoneColouring.resize(boost::extents[map->twoLevel ? 2 : 1][map->width][map->height]);
 }
 
 CMapGenerator::~CMapGenerator()
@@ -81,8 +95,8 @@ void CMapGenerator::initQuestArtsRemaining()
 {
 	for (auto art : VLC->arth->artifacts)
 	{
-		if (art->aClass == CArtifact::ART_TREASURE && art->constituentOf.empty()) //don't use parts of combined artifacts
-		questArtifacts.push_back(art->id);
+		if (art->aClass == CArtifact::ART_TREASURE && VLC->arth->legalArtifact(art->id) && art->constituentOf.empty()) //don't use parts of combined artifacts
+			questArtifacts.push_back(art->id);
 	}
 }
 
@@ -132,7 +146,7 @@ std::string CMapGenerator::getMapDescription() const
 
     std::stringstream ss;
     ss << boost::str(boost::format(std::string("Map created by the Random Map Generator.\nTemplate was %s, Random seed was %d, size %dx%d") +
-        ", levels %s, humans %d, computers %d, water %s, monster %s, second expansion map") % mapGenOptions->getMapTemplate()->getName() %
+        ", levels %s, players %d, computers %d, water %s, monster %s, VCMI map") % mapGenOptions->getMapTemplate()->getName() %
 		randomSeed % map->width % map->height % (map->twoLevel ? "2" : "1") % static_cast<int>(mapGenOptions->getPlayerCount()) %
 		static_cast<int>(mapGenOptions->getCompOnlyPlayerCount()) % waterContentStr[mapGenOptions->getWaterContent()] %
 		monsterStrengthStr[monsterStrengthIndex]);
@@ -157,19 +171,32 @@ std::string CMapGenerator::getMapDescription() const
 void CMapGenerator::addPlayerInfo()
 {
 	// Calculate which team numbers exist
-	std::array<std::list<int>, 2> teamNumbers; // 0= cpu/human, 1= cpu only
+
+	enum ETeams {CPHUMAN = 0, CPUONLY = 1, AFTER_LAST = 2};
+	std::array<std::list<int>, 2> teamNumbers;
+
 	int teamOffset = 0;
-	for(int i = 0; i < 2; ++i)
+	int playerCount = 0;
+	int teamCount = 0;
+
+	for (int i = CPHUMAN; i < AFTER_LAST; ++i)
 	{
-		int playerCount = i == 0 ? mapGenOptions->getPlayerCount() : mapGenOptions->getCompOnlyPlayerCount();
-		int teamCount = i == 0 ? mapGenOptions->getTeamCount() : mapGenOptions->getCompOnlyTeamCount();
+		if (i == CPHUMAN)
+		{
+			playerCount = mapGenOptions->getPlayerCount();
+			teamCount = mapGenOptions->getTeamCount();
+		}
+		else
+		{
+			playerCount = mapGenOptions->getCompOnlyPlayerCount();
+			teamCount = mapGenOptions->getCompOnlyTeamCount();
+		}
 
 		if(playerCount == 0)
 		{
 			continue;
 		}
-		int playersPerTeam = playerCount /
-				(teamCount == 0 ? playerCount : teamCount);
+		int playersPerTeam = playerCount / (teamCount == 0 ? playerCount : teamCount);
 		int teamCountNorm = teamCount;
 		if(teamCountNorm == 0)
 		{
@@ -190,17 +217,23 @@ void CMapGenerator::addPlayerInfo()
 	}
 
 	// Team numbers are assigned randomly to every player
+	//TODO: allow customize teams in rmg template
 	for(const auto & pair : mapGenOptions->getPlayersSettings())
 	{
 		const auto & pSettings = pair.second;
 		PlayerInfo player;
 		player.canComputerPlay = true;
-		int j = pSettings.getPlayerType() == EPlayerType::COMP_ONLY ? 1 : 0;
-		if(j == 0)
+		int j = (pSettings.getPlayerType() == EPlayerType::COMP_ONLY) ? CPUONLY : CPHUMAN;
+		if (j == CPHUMAN)
 		{
 			player.canHumanPlay = true;
 		}
 
+		if (teamNumbers[j].empty())
+		{
+			logGlobal->errorStream() << boost::format("Not enough places in team for %s player") % ((j == CPUONLY) ? "CPU" : "CPU or human");
+			assert (teamNumbers[j].size());
+		}
         auto itTeam = RandomGeneratorUtil::nextItem(teamNumbers[j], rand);
 		player.team = TeamID(*itTeam);
 		teamNumbers[j].erase(itTeam);
@@ -228,28 +261,30 @@ void CMapGenerator::genZones()
 }
 
 void CMapGenerator::fillZones()
-{	
+{
 	//init native town count with 0
 	for (auto faction : VLC->townh->getAllowedFactions())
 		zonesPerFaction[faction] = 0;
 
+	findZonesForQuestArts();
+
 	logGlobal->infoStream() << "Started filling zones";
 
 	//initialize possible tiles before any object is actually placed
 	for (auto it : zones)
-	{
 		it.second->initFreeTiles(this);
-	}
 
-	findZonesForQuestArts();
-	createConnections();
+	createDirectConnections(); //direct
 	//make sure all connections are passable before creating borders
 	for (auto it : zones)
-	{
-		it.second->createBorder(this);
-		 //we need info about all town types to evaluate dwellings and pandoras with creatures properly
+		it.second->createBorder(this); //once direct connections are done
+
+	createConnections2(); //subterranean gates and monoliths
+
+	//we need info about all town types to evaluate dwellings and pandoras with creatures properly
+	for (auto it : zones)
 		it.second->initTownType(this);
-	}
+
 	std::vector<CRmgTemplateZone*> treasureZones;
 	for (auto it : zones)
 	{
@@ -266,7 +301,49 @@ void CMapGenerator::fillZones()
 	createObstaclesCommon2();
 	//place actual obstacles matching zone terrain
 	for (auto it : zones)
+	{
 		it.second->createObstacles2(this);
+	}
+
+	#define PRINT_MAP_BEFORE_ROADS true
+	if (PRINT_MAP_BEFORE_ROADS) //enable to debug
+	{
+		std::ofstream out("road debug");
+		int levels = map->twoLevel ? 2 : 1;
+		int width = map->width;
+		int height = map->height;
+		for (int k = 0; k < levels; k++)
+		{
+			for (int j = 0; j<height; j++)
+			{
+				for (int i = 0; i<width; i++)
+				{
+					char t = '?';
+					switch (getTile(int3(i, j, k)).getTileType())
+					{
+					case ETileType::FREE:
+						t = ' '; break;
+					case ETileType::BLOCKED:
+						t = '#'; break;
+					case ETileType::POSSIBLE:
+						t = '-'; break;
+					case ETileType::USED:
+						t = 'O'; break;
+					}
+
+					out << t;
+				}
+				out << std::endl;
+			}
+			out << std::endl;
+		}
+		out << std::endl;
+	}
+
+	for (auto it : zones)
+	{
+		it.second->connectRoads(this); //draw roads after everything else has been placed
+	}
 
 	//find place for Grail
 	if (treasureZones.empty())
@@ -385,7 +462,7 @@ void CMapGenerator::findZonesForQuestArts()
 	}
 }
 
-void CMapGenerator::createConnections()
+void CMapGenerator::createDirectConnections()
 {
 	for (auto connection : mapGenOptions->getMapTemplate()->getConnections())
 	{
@@ -395,7 +472,6 @@ void CMapGenerator::createConnections()
 		//rearrange tiles in random order
 		auto tilesCopy = zoneA->getTileInfo();
 		std::vector<int3> tiles(tilesCopy.begin(), tilesCopy.end());
-		RandomGeneratorUtil::randomShuffle(tiles, rand);
 
 		int3 guardPos(-1,-1,-1);
 
@@ -403,110 +479,190 @@ void CMapGenerator::createConnections()
 
 		int3 posA = zoneA->getPos();
 		int3 posB = zoneB->getPos();
+		// auto zoneAid = zoneA->getId();
+		auto zoneBid = zoneB->getId();
 
 		if (posA.z == posB.z)
 		{
-			for (auto tile : tiles)
+			std::vector<int3> middleTiles;
+			for (auto tile : tilesCopy)
 			{
 				if (isBlocked(tile)) //tiles may be occupied by subterranean gates already placed
 					continue;
-				foreach_neighbour (tile, [&guardPos, tile, &otherZoneTiles, this](int3 &pos)
+				foreachDirectNeighbour (tile, [&guardPos, tile, &otherZoneTiles, &middleTiles, this, zoneBid](int3 &pos) //must be direct since paths also also generated between direct neighbours
 				{
-					//if (vstd::contains(otherZoneTiles, pos) && !this->isBlocked(pos))
-					if (vstd::contains(otherZoneTiles, pos))
-						guardPos = tile;
+					if (getZoneID(pos) == zoneBid)
+						middleTiles.push_back(tile);
 				});
+			}
+
+			//find tiles with minimum manhattan distance from center of the mass of zone border
+			size_t tilesCount = middleTiles.size() ? middleTiles.size() : 1;
+			int3 middleTile = std::accumulate(middleTiles.begin(), middleTiles.end(), int3(0, 0, 0));
+			middleTile.x /= tilesCount;
+			middleTile.y /= tilesCount;
+			middleTile.z /= tilesCount; //TODO: implement division operator for int3?
+			boost::sort(middleTiles, [middleTile](const int3 &lhs, const int3 &rhs) -> bool
+			{
+				//choose tiles with both corrdinates in the middle
+				return lhs.mandist2d(middleTile) < rhs.mandist2d(middleTile);
+			});
+
+			//remove 1/4 tiles from each side - path should cross zone borders at smooth angle
+			size_t removedCount = tilesCount / 4; //rounded down
+			middleTiles.erase(middleTiles.end() - removedCount, middleTiles.end());
+			middleTiles.erase(middleTiles.begin(), middleTiles.begin() + removedCount);
+
+			RandomGeneratorUtil::randomShuffle(middleTiles, rand);
+			for (auto tile : middleTiles)
+			{
+				guardPos = tile;
 				if (guardPos.valid())
 				{
-					setOccupied (guardPos, ETileType::FREE); //just in case monster is too weak to spawn
-					zoneA->addMonster (this, guardPos, connection.getGuardStrength(), false, true);
+					setOccupied(guardPos, ETileType::FREE); //just in case monster is too weak to spawn
+					zoneA->addMonster(this, guardPos, connection.getGuardStrength(), false, true);
 					//zones can make paths only in their own area
-					zoneA->crunchPath (this, guardPos, posA, zoneA->getId(), zoneA->getFreePaths()); //make connection towards our zone center
-					zoneB->crunchPath (this, guardPos, posB, zoneB->getId(), zoneB->getFreePaths()); //make connection towards other zone center
+					zoneA->crunchPath(this, guardPos, posA, true, zoneA->getFreePaths()); //make connection towards our zone center
+					zoneB->crunchPath(this, guardPos, posB, true, zoneB->getFreePaths()); //make connection towards other zone center
+
+					zoneA->addRoadNode(guardPos);
+					zoneB->addRoadNode(guardPos);
 					break; //we're done with this connection
 				}
 			}
 		}
-		else //create subterranean gates between two zones
-		{	
-			//find point on the path between zones
-			float3 offset (posB.x - posA.x, posB.y - posA.y, 0);
 
-			float distance = posB.dist2d(posA);
-			vstd::amax (distance, 0.5f);
-			offset /= distance; //get unit vector
-			float3 vec (0, 0, 0);
-			//use reduced size of underground zone - make sure gate does not stand on rock
-			int3 tile = posA;
-			int3 otherTile = tile;
+		if (!guardPos.valid())
+			connectionsLeft.push_back(connection);
+	}
+}
 
-			bool stop = false;
-			while (!stop)
+void CMapGenerator::createConnections2()
+{
+	for (auto & connection : connectionsLeft)
+	{
+		auto zoneA = connection.getZoneA();
+		auto zoneB = connection.getZoneB();
+
+		int3 guardPos(-1, -1, -1);
+
+		int3 posA = zoneA->getPos();
+		int3 posB = zoneB->getPos();
+
+		auto strength = connection.getGuardStrength();
+
+		if (posA.z != posB.z) //try to place subterranean gates
+		{
+			auto sgt = VLC->objtypeh->getHandlerFor(Obj::SUBTERRANEAN_GATE, 0)->getTemplates().front();
+			auto tilesBlockedByObject = sgt.getBlockedOffsets();
+
+			auto factory = VLC->objtypeh->getHandlerFor(Obj::SUBTERRANEAN_GATE, 0);
+			auto gate1 = factory->create(ObjectTemplate());
+			auto gate2 = factory->create(ObjectTemplate());
+
+			while (!guardPos.valid())
 			{
-				vec += offset; //this vector may extend beyond line between zone centers, in case they are directly over each other
-				tile = posA + int3(vec.x, vec.y, 0);
-				float distanceFromA = posA.dist2d(tile);
-				float distanceFromB = posB.dist2d(tile);
+				bool continueOuterLoop = false;
+				//find common tiles for both zones
+				auto tileSetA = zoneA->getPossibleTiles(),
+					tileSetB = zoneB->getPossibleTiles();
 
-				if (distanceFromA + distanceFromB > std::max<int>(zoneA->getSize() + zoneB->getSize(), distance))
-					break; //we are too far away to ever connect
+				std::vector<int3> tilesA(tileSetA.begin(), tileSetA.end()),
+					tilesB(tileSetB.begin(), tileSetB.end());
 
-				//if zone is underground, gate must fit within its (reduced) radius
-				if (distanceFromA > 5 && (!posA.z || distanceFromA < zoneA->getSize() - 3) &&
-					distanceFromB > 5 && (!posB.z || distanceFromB < zoneB->getSize() - 3))
+				std::vector<int3> commonTiles;
+
+				//required for set_intersection
+				boost::sort(tilesA);
+				boost::sort(tilesB);
+
+				boost::set_intersection(tilesA, tilesB, std::back_inserter(commonTiles), [](const int3 &lhs, const int3 &rhs) -> bool
 				{
-					otherTile = tile;
+					//ignore z coordinate
+					if (lhs.x < rhs.x)
+						return true;
+					else
+						return lhs.y < rhs.y;
+				});
+
+				vstd::erase_if(commonTiles, [](const int3 &tile) -> bool
+				{
+					return (!tile.x) || (!tile.y); //gates shouldn't go outside map (x = 0) and look bad at the very top (y = 0)
+				});
+
+				if (commonTiles.empty())
+					break; //nothing more to do
+
+				boost::sort(commonTiles, [posA, posB](const int3 &lhs, const int3 &rhs) -> bool
+				{
+					//choose tiles which are equidistant to zone centers
+					return (std::abs<double>(posA.dist2dSQ(lhs) - posB.dist2dSQ(lhs)) < std::abs<double>((posA.dist2dSQ(rhs) - posB.dist2dSQ(rhs))));
+				});
+
+				for (auto tile : commonTiles)
+				{
+					tile.z = posA.z;
+					int3 otherTile = tile;
 					otherTile.z = posB.z;
 
-					if (vstd::contains(tiles, tile) && vstd::contains(otherZoneTiles, otherTile))
+					float distanceFromA = posA.dist2d(tile);
+					float distanceFromB = posB.dist2d(otherTile);
+
+					if (distanceFromA > 5 && distanceFromB > 5)
 					{
-						bool withinZone = true;
-
-						foreach_neighbour (tile, [&withinZone, &tiles](int3 &pos)
+						if (zoneA->areAllTilesAvailable(this, gate1, tile, tilesBlockedByObject) &&
+							zoneB->areAllTilesAvailable(this, gate2, otherTile, tilesBlockedByObject))
 						{
-							if (!vstd::contains(tiles, pos))
-								withinZone = false;
-						});
-						foreach_neighbour (otherTile, [&withinZone, &otherZoneTiles](int3 &pos)
-						{
-							if (!vstd::contains(otherZoneTiles, pos))
-								withinZone = false;
-						});
+							if (zoneA->getAccessibleOffset(this, sgt, tile).valid() && zoneB->getAccessibleOffset(this, sgt, otherTile).valid())
+							{
+								EObjectPlacingResult::EObjectPlacingResult result1 = zoneA->tryToPlaceObjectAndConnectToPath(this, gate1, tile);
+								EObjectPlacingResult::EObjectPlacingResult result2 = zoneB->tryToPlaceObjectAndConnectToPath(this, gate2, otherTile);
 
-						if (withinZone)
-						{
-							auto gate1 = new CGSubterraneanGate;
-							gate1->ID = Obj::SUBTERRANEAN_GATE;
-							gate1->subID = 0;
-							zoneA->placeAndGuardObject(this, gate1, tile, connection.getGuardStrength());
-							auto gate2 = new CGSubterraneanGate(*gate1);
-							zoneB->placeAndGuardObject(this, gate2, otherTile, connection.getGuardStrength());
-
-							stop = true; //we are done, go to next connection
+								if ((result1 == EObjectPlacingResult::SUCCESS) && (result2 == EObjectPlacingResult::SUCCESS))
+								{
+									zoneA->placeObject(this, gate1, tile);
+									zoneA->guardObject(this, gate1, strength, true, true);
+									zoneB->placeObject(this, gate2, otherTile);
+									zoneB->guardObject(this, gate2, strength, true, true);
+									guardPos = tile; //set to break the loop
+									break;
+								}
+								else if ((result1 == EObjectPlacingResult::SEALED_OFF) || (result2 == EObjectPlacingResult::SEALED_OFF))
+								{
+									//sealed-off tiles were blocked, exit inner loop and get another tile set
+									continueOuterLoop = true;
+									break;
+								}
+								else
+									continue; //try with another position
+							}
 						}
 					}
 				}
+				if (!continueOuterLoop) //we didn't find ANY tile - break outer loop			
+					break;
+			}
+			if (!guardPos.valid()) //cleanup? is this safe / enough?
+			{
+				delete gate1;
+				delete gate2;
 			}
-			if (stop)
-				continue;
 		}
 		if (!guardPos.valid())
 		{
-			auto teleport1 = new CGMonolith;
-			teleport1->ID = Obj::MONOLITH_TWO_WAY;
-			teleport1->subID = getNextMonlithIndex();
+			auto factory = VLC->objtypeh->getHandlerFor(Obj::MONOLITH_TWO_WAY, getNextMonlithIndex());
+			auto teleport1 = factory->create(ObjectTemplate());
+			auto teleport2 = factory->create(ObjectTemplate());
 
-			auto teleport2 = new CGMonolith(*teleport1);
-
-			zoneA->addRequiredObject (teleport1, connection.getGuardStrength());
-			zoneB->addRequiredObject (teleport2, connection.getGuardStrength());
-		}		
+			zoneA->addRequiredObject(teleport1, strength);
+			zoneB->addRequiredObject(teleport2, strength);
+		}
 	}
 }
 
 void CMapGenerator::addHeaderInfo()
 {
-	map->version = EMapFormat::SOD;
+	map->version = EMapFormat::VCMI;
 	map->width = mapGenOptions->getWidth();
 	map->height = mapGenOptions->getHeight();
 	map->twoLevel = mapGenOptions->getHasTwoLevels();
@@ -516,6 +672,13 @@ void CMapGenerator::addHeaderInfo()
 	addPlayerInfo();
 }
 
+void CMapGenerator::checkIsOnMap(const int3& tile) const
+{
+	if (!map->isInTheMap(tile))
+		throw  rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+}
+
+
 std::map<TRmgTemplateZoneId, CRmgTemplateZone*> CMapGenerator::getZones() const
 {
 	return zones;
@@ -523,67 +686,99 @@ std::map<TRmgTemplateZoneId, CRmgTemplateZone*> CMapGenerator::getZones() const
 
 bool CMapGenerator::isBlocked(const int3 &tile) const
 {
-	if (!map->isInTheMap(tile))
-		throw  rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	return tiles[tile.x][tile.y][tile.z].isBlocked();
 }
 bool CMapGenerator::shouldBeBlocked(const int3 &tile) const
 {
-	if (!map->isInTheMap(tile))
-		throw rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	return tiles[tile.x][tile.y][tile.z].shouldBeBlocked();
 }
 bool CMapGenerator::isPossible(const int3 &tile) const
 {
-	if (!map->isInTheMap(tile))
-		throw rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	return tiles[tile.x][tile.y][tile.z].isPossible();
 }
 bool CMapGenerator::isFree(const int3 &tile) const
 {
-	if (!map->isInTheMap(tile))
-		throw rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	return tiles[tile.x][tile.y][tile.z].isFree();
 }
 bool CMapGenerator::isUsed(const int3 &tile) const
 {
-	if (!map->isInTheMap(tile))
-		throw  rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	return tiles[tile.x][tile.y][tile.z].isUsed();
 }
+
+bool CMapGenerator::isRoad(const int3& tile) const
+{
+	checkIsOnMap(tile);
+
+	return tiles[tile.x][tile.y][tile.z].isRoad();
+}
+
 void CMapGenerator::setOccupied(const int3 &tile, ETileType::ETileType state)
 {
-	if (!map->isInTheMap(tile))
-		throw rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	tiles[tile.x][tile.y][tile.z].setOccupied(state);
 }
 
-CTileInfo CMapGenerator::getTile(const int3&  tile) const
+void CMapGenerator::setRoad(const int3& tile, ERoadType::ERoadType roadType)
 {
-	if (!map->isInTheMap(tile))
-		throw rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
+
+	tiles[tile.x][tile.y][tile.z].setRoadType(roadType);
+}
+
+
+CTileInfo CMapGenerator::getTile(const int3& tile) const
+{
+	checkIsOnMap(tile);
 
 	return tiles[tile.x][tile.y][tile.z];
 }
 
+TRmgTemplateZoneId CMapGenerator::getZoneID(const int3& tile) const
+{
+	checkIsOnMap(tile);
+
+	return zoneColouring[tile.z][tile.x][tile.y];
+}
+
+void CMapGenerator::setZoneID(const int3& tile, TRmgTemplateZoneId zid) 
+{
+	checkIsOnMap(tile);
+
+	zoneColouring[tile.z][tile.x][tile.y] = zid;
+}
+
+bool CMapGenerator::isAllowedSpell(SpellID sid) const
+{
+	assert(sid >= 0);
+	if (sid < map->allowedSpell.size())
+	{
+		return map->allowedSpell[sid];
+	}
+	else
+		return false;
+}
+
 void CMapGenerator::setNearestObjectDistance(int3 &tile, float value)
 {
-	if (!map->isInTheMap(tile))
-		throw rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	tiles[tile.x][tile.y][tile.z].setNearestObjectDistance(value);
 }
 
 float CMapGenerator::getNearestObjectDistance(const int3 &tile) const
 {
-	if (!map->isInTheMap(tile))
-		throw rmgException(boost::to_string(boost::format("Tile %s is outside the map") % tile));
+	checkIsOnMap(tile);
 
 	return tiles[tile.x][tile.y][tile.z].getNearestObjectDistance();
 }
